#!/usr/bin/env python

################################################################################
# aggregateFASTAfiles.py: Collect the set of input sequences from the
# file names in sampleDesc.csv, and aggregate into a single FASTA
# file.  
#
# Within an input sample, identical sequences are aggregated into a
# single FASTA sequence with cnt:<int> representing the number of
# those identical sequences.
################################################################################

import sys
import csv
import pdb
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord

def main():
    if len(sys.argv) != 3:
        print >> sys.stderr, \
           """usage: aggregateFASTAfiles.py sampleDesc.csv output.fa"""
        sys.exit(1)
    sampleDescFn = sys.argv[1]
    outFn = sys.argv[2]
    outH = open(outFn, "w")

# read and sanity check sampleDesc.csv 
    sampleDescList = [d for d in csv.DictReader(open(sampleDescFn, 'rb'))]

    # make sure short names are unique
    assert (len(set([sd['shortSampleName'] for sd in sampleDescList])) == len(sampleDescList))

    for sd in sampleDescList:
        fn = sd['inputFastaFileName'] 
        sn = sd['shortSampleName'] 
        print 'Reading %s, saving as short name %s' % (fn, sn)

        # collect & count identical sequences
        bag = {}                # bag of DNA sequences, value is cnt
        for inRec in SeqIO.parse(open(fn, 'rb'), "fasta"):
            seq = str(inRec.seq)
            bag[seq] = bag.setdefault(seq, 0) + 1    

        # generate output.  Don't use BioPython SeqIO because
        # HMM-FRAME needs the sequences to be on a single long line
        for outSeq in bag:
            outH.write('>file:%s|cnt:%d\n%s\n' % (sn, bag[outSeq], outSeq))

if __name__ == '__main__':
    main()
    

