#!/usr/bin/env python

################################################################################
# countSeqs.py: Utility to count sequences in fasta files. 
#
# FASTA headers must contain file:<alphanumeric> and cnt:<num> fields.
# Other fields are ignored.
# 
# Example:
# >file:mp1|cnt:2320|hmm_name:Cytochrom_D1|score:124.361|error_num:2
################################################################################

import sys
import pdb
from Bio import SeqIO
import re

reGetCnt = re.compile('cnt:(\d+)')
reGetFile = re.compile('file:(\w+)')

def parseFile(inpFAheader):
    hit = reGetFile.search(inpFAheader)
    if hit == None:
        print >>sys.stderr, "file:<name> field missing in header:\n>%s" % (inpFAheader)
        sys.exit(1)
    return hit.group(1)

def parseCnt(inpFAheader):
    hit = reGetCnt.search(inpFAheader)
    if hit == None:
        print >>sys.stderr, "cnt:<num> field missing in header:\n>%s" % (inpFAheader)
        sys.exit(1)
    return int(hit.group(1))

def main():
    if len(sys.argv) != 2:
        print >> sys.stderr, \
           """usage: countSeqs.py file.fa
              file.fa must contain headers with file:<alphanumeric> and cnt:<num> fields"""
        sys.exit(1)

    inFn = sys.argv[1]
    print
    print inFn
    print

    bag = {}   # keyed by <file>, value is total <cnt>
    for sr in SeqIO.parse(inFn, "fasta"):
        faHdr = sr.description
        file = parseFile (faHdr)
        cnt = parseCnt (faHdr)
        bag[file] = bag.setdefault(file, 0) + cnt
    
    print "file,totSeqs"
    for k in sorted(bag):
        print "%s,%d" % (k, bag[k]) # csv-friendly

    print

if __name__ == '__main__':
    main()
