#!/usr/bin/env python

################################################################################
# Read the HMMFRAME output and write only sequences above the
# user-specified score threshold
################################################################################

import sys
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
import re
import numpy

def processInput(minHMMFRAMEscore):
    reGetScore = re.compile('score:([\d\.-]+)')
    reGetCnt = re.compile('cnt:([\d\.-]+)')

    totalOutputSeqs = 0

    allScores = []
    for record in SeqIO.parse(sys.stdin, "fasta"):
        hmmframeScore = float(reGetScore.search(record.description).group(1))
        seqCnt = int(reGetCnt.search(record.description).group(1))

        # for histogram, stats
        allScores += [hmmframeScore]*seqCnt

        if hmmframeScore >= minHMMFRAMEscore:
            SeqIO.write(record, sys.stdout, "fasta")
            totalOutputSeqs += seqCnt

    hist, bin_edges = numpy.histogram(allScores, bins=15)
    print >>sys.stderr, 'Histogram of HMMFRAME scores\n\nScore:              count (cumulative %)'
    cumulativeTot = 0
    for i in range(len(hist)):
        cumulativeTot += hist[i]
        cumulativePercent = 100.0 * cumulativeTot / len(allScores)
        rangeStr = '[%.2f,%.2f)' % (bin_edges[i], bin_edges[i+1])
        print >>sys.stderr, '%-18s %6d (%4.1f%%)' %\
              (rangeStr, hist[i], cumulativePercent)

    print >>sys.stderr, \
        "\nTotal input sequences: %d, total output sequences: %d (minHMMFRAMEscore=%.2f)" %\
        (len(allScores), totalOutputSeqs, minHMMFRAMEscore)


def main():

    if len(sys.argv) <> 2:
        print >>sys.stderr, "Usage: filterHMMFRAMEscores.py minHMMFRAMEscore\n(reads stdin, writes stdout)"
        sys.exit(1)

    minHMMFRAMEscore = float(sys.argv[1])
    processInput(minHMMFRAMEscore)

if __name__ == '__main__':
    main()
